<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class StandWithUkraineForm.
 *
 * @package Drupal\standwithukraine\Form
 */
class StandWithUkraineForm extends ConfigFormBase {

  /**
   * The suggested websites list.
   */
  const URLS = [
    'https://supportukrainenow.org',
    'https://war.ukraine.ua/',
  ];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The favicon.
   *
   * @var \Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface
   */
  protected $favicon;

  /**
   * The time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->cache = $container->get('cache.default');
    $instance->favicon = $container->get('standwithukraine.favicon');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['standwithukraine.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'standwithukraine_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Link'),
      '#description' => sprintf(
        '%s %s.',
        $this->t('For example:'),
        implode(
          ', ',
          array_map(
            function ($url) {
              return Link::fromTextAndUrl(
                parse_url($url, PHP_URL_HOST),
                Url::fromUri($url, ['target' => '_blank'])
              )->toString();
            },
            self::URLS
          )
        )
      ),
      '#default_value' => $config->get('url'),
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#description' => $this->t('Select on which side of the page a message
should be displayed.'),
      '#options' => [
        'left' => $this->t('Left'),
        '' => $this->t('Top (default)'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => $config->get('position') ?? '',
    ];

    $form['offset'] = [
      '#type' => 'number',
      '#title' => $this->t('Offset'),
      '#description' => $this->t('How many pixels down the message should be
shifted down the page. When a field is empty then the message will be shifted
down by 20 percent from the page height. Available only for the left or right
position of the message.'),
      '#min' => 0,
      '#field_suffix' => $this->t('pixels'),
      '#default_value' => $config->get('offset'),
      '#states' => [
        'enabled' => [
          ':input[name="position"]' => [
            ['value' => 'left'],
            ['value' => 'right'],
          ],
        ],
      ],
    ];

    $form['favicon'] = [
      '#type' => 'details',
      '#title' => 'Favicon',
      '#tree' => TRUE,
      '#open' => FALSE,
    ];

    $form['favicon']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#default_value' => !$config->get('favicon.disable'),
    ];

    $form['favicon']['ratios'] = [
      '#type' => 'details',
      '#title' => $this->t('Ratios'),
      '#description' => $this->t('Define how many percentages of icon height
should be dedicated to blue color.'),
      '#open' => FALSE,
      '#states' => [
        'disabled' => [
          ':input[name="favicon[enable]"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $theme_config = $this->config('system.theme');
    $ratios = (array) $config->get('favicon.ratios');
    $suffix = '?timestamp=' . $this->time->getRequestTime();
    $count = 0;

    $theme_types = [
      'default' => $this->t('Default theme'),
      'admin' => $this->t('Admin theme'),
    ];

    foreach ($theme_types as $theme_type => $title) {
      $theme_name = $theme_config->get($theme_type);

      if (empty($theme_name)) {
        continue;
      }

      $favicon = $this->favicon->build($theme_name);

      if (!isset($favicon[$theme_name])) {
        continue;
      }

      $count++;

      if (isset($ratios[$theme_name])) {
        $ratio = (int) $ratios[$theme_name];
      }
      elseif (in_array($theme_name, StandWithUkraineFaviconInterface::THEMES)) {
        $ratio = StandWithUkraineFaviconInterface::DEFINED_RATIO;
      }
      else {
        $ratio = StandWithUkraineFaviconInterface::UNDEFINED_RATIO;
      }

      $custom = $ratio !== StandWithUkraineFaviconInterface::UNDEFINED_RATIO;

      $form['favicon']['#open'] |= $custom;
      $form['favicon']['ratios']['#open'] |= $custom;

      $form['favicon']['ratios'][$theme_name] = [
        '#type' => 'details',
        '#title' => $title,
        '#open' => $custom,
      ];

      foreach ($favicon[$theme_name] as $dimensions => $url) {
        $form['favicon']['ratios'][$theme_name][$dimensions] = [
          '#type' => 'html_tag',
          '#tag' => 'img',
          '#attributes' => [
            'src' => $url . $suffix,
          ],
        ];
      }

      $form['favicon']['ratios'][$theme_name]['ratio'] = [
        '#type' => 'number',
        '#min' => 10,
        '#max' => 90,
        '#default_value' => $ratio,
        '#field_suffix' => '%',
      ];
    }

    $form['favicon']['#open'] &= $form['favicon']['enable']['#default_value'];
    $form['favicon']['#access'] = $count > 0;

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    foreach (['url', 'position', 'offset'] as $key) {
      $config->set($key, $form_state->getValue($key));
    }

    $config->set(
      'favicon.disable',
      !$form_state->getValue(['favicon', 'enable'])
    );

    foreach (Element::children($form['favicon']['ratios']) as $theme_name) {
      $config->set(
        'favicon.ratios.' . $theme_name,
        $form_state->getValue(['favicon', 'ratios', $theme_name, 'ratio'])
      );
    }

    $config->save();

    $this->cache->invalidate('standwithukraine');

    parent::submitForm($form, $form_state);
  }

}
